﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.IO;

namespace Pumatronix_Sample
{
    public partial class Principal : Form
    {
        // ID da conexão - IMPORTANTE!
        private int id;

        // Constantes para a imagem
        private const int IMAGE_JPEG_SIZE = 2 * 1024 * 1024;            // Tamanho da imagem JPEG, com sobra (dobro da resolução máxima)
        private const int IMAGE_BITMAP_SIZE = 2457666;                  // Tamanho com sobra para bitmap
        private const int FORMAT_JPEG = 1;                              // Argumento para requisitar JPEG
        private const int FORMAT_BMP = 0;                               // Argumento para requisitar BMP

        // Threads
        private bool toKillVideo = false;                               // Para matar a thread de requisição de video
        private bool toKillFps = false;                                 // Para matar a thread de contagem de FPS
        private int fpsCount = 0;                                       // Contagem de FPS
        private Mutex fpsMutex = new Mutex(false);                      // Mutex para acesso à contagem de FPS

        public Principal()
        {
            InitializeComponent();
            // Inicializa icone do programa
            this.Icon = Properties.Resources.pumalogo;
            // Threads podem manipular UI
            CheckForIllegalCrossThreadCalls = false;
        }

        #region Funções relacionadas ao Form
        /// <summary>
        /// Clique em Listar Câmeras
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bListarCameras_Click(object sender, EventArgs e)
        {
            // Limpa list box e desabilita botão
            lbListaCameras.Items.Clear();
            bListarCameras.Enabled = false;

            // Aloca vetor para resposta
            byte[] lista = new byte[500];
            // Requisita broadcast para listagem de câmeras
            ItsCamDLL.broadcast(lista, lista.Length);

            // Transforma os bytes em UTF8
            string strIps = System.Text.Encoding.UTF8.GetString(lista);
            // Lista não vazia
            if (strIps[0].ToString() != "\0")
            {
                // Recorta da maneira correta
                string[] ips = strIps.Substring(0, strIps.LastIndexOf(";")).Split(";".ToCharArray());
                // Popula o list box com os IPs
                foreach (string ip in ips)
                    lbListaCameras.Items.Add(ip);
            }

            // Habilita botão
            bListarCameras.Enabled = true;
        }
        /// <summary>
        /// Usuário selecionou algum item da list box dos IPs, copia para a text box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void lbListaCameras_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Copia IP para o text box
            tbIP.Text = lbListaCameras.Text;
        }
        /// <summary>
        /// Algo foi escrito na text box, checa se é IP válido
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tbIP_TextChanged(object sender, EventArgs e)
        {
            // Pelo menos 7 caracteres (0.0.0.0)
            if (tbIP.Text.Length >= 7)
                // Testa
                bConectar.Enabled = checkIp(tbIP.Text);
            else
                bConectar.Enabled = false;
        }
        /// <summary>
        /// Clicou no botão de conectar - conecta ou desconecta da câmera desejada
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bConectar_Click(object sender, EventArgs e)
        {
            if (bConectar.Text == "Conectar")
            {
                // Desabilita text box, botão e altera botão
                tbIP.Enabled = false;
                lbListaCameras.Enabled = false;
                bListarCameras.Enabled = false;
                bConectar.Enabled = false;
                bConectar.Text = "Desconectar";

                pbFoto.Image = null;

                string ip = tbIP.Text;
                // Tenta conectar
                id = ItsCamDLL.criarConexaoItscam300((ip + "\0").ToCharArray());
                if (id < 0)
                {
                    // Não conseguiu... arruma parte visual e sai
                    MessageBox.Show("Não foi possível conectar!", "Erro", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    tbIP.Enabled = true;
                    lbListaCameras.Enabled = true;
                    bListarCameras.Enabled = true;
                    bConectar.Enabled = true;
                    bConectar.Text = "Conectar";
                }
                else
                {
                    // Conectou, arruma parte visual e continua
                    bConectar.Enabled = true;
                    gbConfiguracoes.Visible = true;
                    gbImagemVideo.Visible = true;

                    refreshCamConfig();
                }
            }
            else
            {
                if (bVideo.Text == "Capturar Vídeo")
                {
                    // Mata threads
                    toKillVideo = true;
                    toKillFps = true;

                    // Encerra conexão
                    if (ItsCamDLL.fecharConexaoItscam300(id) != 1)
                        MessageBox.Show("Não foi possível desconectar!", "Erro", MessageBoxButtons.OK, MessageBoxIcon.Error);

                    tbIP.Enabled = true;
                    lbListaCameras.Enabled = true;
                    bListarCameras.Enabled = true;
                    bConectar.Enabled = true;
                    bConectar.Text = "Conectar";
                    gbConfiguracoes.Visible = false;
                    gbImagemVideo.Visible = false;
                }
                else
                {
                    DialogResult result = MessageBox.Show("A captura de vídeo deve ser suspensa antes de desconectar. Deseja parar a captura de vídeo e desconectar?",
                                                           "Desconectar", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    if(result == System.Windows.Forms.DialogResult.Yes)
                    {
                        // Mata threads
                        toKillVideo = true;
                        toKillFps = true;
                        // Aguarda morrer
                        Thread.Sleep(500);

                        // Encerra conexão
                        if (ItsCamDLL.fecharConexaoItscam300(id) != 1)
                            MessageBox.Show("Não foi possível desconectar!", "Erro", MessageBoxButtons.OK, MessageBoxIcon.Error);

                        tbIP.Enabled = true;
                        lbListaCameras.Enabled = true;
                        bListarCameras.Enabled = true;
                        bConectar.Enabled = true;
                        bConectar.Text = "Conectar";
                        gbConfiguracoes.Visible = false;
                        gbImagemVideo.Visible = false;
                        bVideo.Text = "Capturar Vídeo";
                        bFoto.Enabled = true;
                    }
                }
            }
        }
        /// <summary>
        /// Requisitou imagem
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bFoto_Click(object sender, EventArgs e)
        {
            // Requisição de foto

            // Cria buffer para a imagem, considerando BMP (bem maior)
            byte[] buf = new byte[IMAGE_BITMAP_SIZE];
            int formato, qualidade;
            // Checa a opção do usuário
            if (rbBmp.Checked)
            {
                // BMP
                formato = FORMAT_BMP;
                qualidade = 0;
            }
            else
            {
                // JPEG
                formato = FORMAT_JPEG;
                qualidade = Convert.ToInt32(mtbQualidadeJpeg.Text);
            }

            // Requisita imagem
            int size = ItsCamDLL.requisitaFoto(id, buf, formato, qualidade);
            if (size < 0)
            {
                // Erro ao requisitar
                MessageBox.Show("Não foi possível requisitar uma imagem.", "Erro ao requisitar imagem", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            // Cria memory stream para converter o buffer da imagem
            MemoryStream ms = new MemoryStream(buf);
            // Cria imagem
            Image img = Image.FromStream(ms);

            pbFoto.Image = img;
        }
        /// <summary>
        /// Requisita captura de vídeo
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bVideo_Click(object sender, EventArgs e)
        {
            // Lê trigger
            int trigger = ItsCamDLL.leTrigger(id);
            if ((trigger == 5 || trigger == 6 || trigger == 7 || trigger == 8) && bVideo.Text == "Capturar Vídeo")
                if (MessageBox.Show("Com o trigger configurado para aproximação, afastamento ou contínuo o desempenho da captura de vídeo" +
                    " será muito baixo. Deseja continuar?", "Captura de Vídeo", MessageBoxButtons.YesNo) == DialogResult.No)
                    return;

            if (bVideo.Text == "Capturar Vídeo")
            {
                // Vai começar a captura
                bFoto.Enabled = false;
                bVideo.Text = "Parar Captura";

                // Inicia threads
                toKillVideo = false;
                toKillFps = false;
                fpsCount = 0;
                Thread video = new Thread(capturaVideo);
                Thread fps = new Thread(contaFPS);
                video.IsBackground = true;
                fps.IsBackground = true;
                video.Start();
                fps.Start();
            }
            else
            {
                // Vai parar a captura
                toKillVideo = true;
                toKillFps = true;
                // Aguarda encerrar
                Thread.Sleep(500);
                lFps.Text = "";

                pbFoto.Image = null;
                bFoto.Enabled = true;
                bVideo.Text = "Capturar Vídeo";
            }
        }
        /// <summary>
        /// Atualizar configurações da câmera
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bAtualizar_Click(object sender, EventArgs e)
        {
            refreshCamConfig();
        }
        /// <summary>
        /// Salva as configurações de rede da câmera
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bSalvarRede_Click(object sender, EventArgs e)
        {
            string ip = tbIpAtual.Text;
            string mascara = tbMascaraRede.Text;
            string gateway = tbGateway.Text;

            // Se IPs forem válidos, altera configuração
            if (checkIp(ip) && checkIp(mascara) && checkIp(gateway))
            {
                DialogResult result = MessageBox.Show("Deseja realmente alterar as configurações de rede para:\nIP: " + ip + "\nMáscara: " + mascara + "\nGateway: " + gateway + " ?",
                                                      "Alteração de configurações de rede", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);

                if (result == DialogResult.Yes)
                {
                    if (ItsCamDLL.setaIp(id, ip.ToCharArray()) != 1 ||
                        ItsCamDLL.setaMascaraRede(id, mascara.ToCharArray()) != 1 ||
                        ItsCamDLL.setaGateway(id, gateway.ToCharArray()) != 1)
                    {
                        MessageBox.Show("Não foi possível alterar a configuração de rede.", "Erro ao alterar configuração", MessageBoxButtons.OK, MessageBoxIcon.Information);
                        refreshCamConfig();
                        return;
                    }
                    else
                        MessageBox.Show("As configurações de rede somente serão alteradas após a câmera ser reiniciada!", "Atenção", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            else
                MessageBox.Show("Configuração de rede inválida.", "Erro", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
        /// <summary>
        /// Alterou configuração da autoíris, manda câmera alterar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rbSemAutoiris_CheckedChanged(object sender, EventArgs e)
        {
            int autoiris;
            if (rbSemAutoiris.Checked)
                // Sem autoíris
                autoiris = 0;
            else
                // Com autoíris
                autoiris = 1;

            // Seta autoíris
            if (ItsCamDLL.setaLenteAutoIris(id, autoiris) != 1)
            {
                MessageBox.Show("Não foi possível alterar a configuração da autoíris.", "Erro ao alterar configuração", MessageBoxButtons.OK, MessageBoxIcon.Information);
                refreshCamConfig();
            }
        }
        /// <summary>
        /// Alterou configuração modo day/night, manda a câmera alterar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rbDayNightAutomatico_CheckedChanged(object sender, EventArgs e)
        {
            int daynight;

            if (rbDayNightAutomatico.Checked)
                // Automático
                daynight = 0;
            else if (rbDay.Checked)
                // Modo day
                daynight = 1;
            else
                // Modo night
                daynight = 2;

            // Seta modo day/night
            if (ItsCamDLL.setaModoDayNight(id, daynight) != 1)
            {
                MessageBox.Show("Não foi possível alterar o modo day/night.", "Erro ao alterar configuração", MessageBoxButtons.OK, MessageBoxIcon.Information);
                refreshCamConfig();
            }
        }
        /// <summary>
        /// Alterou configuração da rotação da image, manda a câmera alterar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void rbSemRotacao_CheckedChanged(object sender, EventArgs e)
        {
            int rotacao;

            if (rbSemRotacao.Checked)
                // Sem rotação
                rotacao = 0;
            else
                // Com rotação
                rotacao = 1;

            // Seta rotação
            if (ItsCamDLL.setaRotacao(id, rotacao) != 1)
            {
                MessageBox.Show("Não foi possível alterar a rotação da imagem.", "Erro ao alterar configuração", MessageBoxButtons.OK, MessageBoxIcon.Information);
                refreshCamConfig();
            }
        }
        #endregion

        #region Threads
        /// <summary>
        /// Thread que captura quadro do vídeo e atualiza a imagem
        /// </summary>
        private void capturaVideo()
        {
            while(!toKillVideo)
            {
                // Cria buffer para a imagem, considerando BMP (bem maior)
                byte[] buf = new byte[IMAGE_BITMAP_SIZE];
                int formato, qualidade;
                // Checa a opção do usuário
                if (rbBmp.Checked)
                {
                    // BMP
                    formato = FORMAT_BMP;
                    qualidade = 0;
                }
                else
                {
                    // JPEG
                    formato = FORMAT_JPEG;
                    qualidade = Convert.ToInt32(mtbQualidadeJpeg.Text);
                }

                // Requisita imagem
                int size = ItsCamDLL.requisitaQuadroVideo(id, buf, formato, qualidade);
                if (size < 0)
                {
                    // Erro ao requisitar
                    MessageBox.Show("Não foi possível requisitar uma imagem.", "Erro ao requisitar imagem", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                // Cria memory stream para converter o buffer da imagem
                MemoryStream ms = new MemoryStream(buf);
                // Cria imagem
                Image img = Image.FromStream(ms);

                // Incrementa contagem de FPS
                fpsMutex.WaitOne();
                fpsCount++;
                fpsMutex.ReleaseMutex();

                // Atualiza imagem
                pbFoto.Image = img;
            }
        }
        /// <summary>
        /// Thread que calcula o FPS atual
        /// </summary>
        private void contaFPS()
        {
            int lastSec = 0;
            while(!toKillFps)
            {
                // Já deu 1 segundo?
                if(lastSec != DateTime.Now.Second && !toKillFps)
                {
                    // Atualiza FPS
                    fpsMutex.WaitOne();
                    lFps.Text = "FPS: " + fpsCount.ToString();
                    fpsCount = 0;
                    fpsMutex.ReleaseMutex();
                    // Atualiza segundo
                    lastSec = DateTime.Now.Second;
                }
            }
        }
        #endregion

        #region Diversas
        /// <summary>
        /// Realiza a leitura das configurações da câmera e atualiza os valores
        /// </summary>
        private void refreshCamConfig()
        {
            // Inicializa variáveis
            int versao, revisao, autoiris, daynight, rotacao;
            StringBuilder mac = new StringBuilder(20);
            StringBuilder ip = new StringBuilder(17);
            StringBuilder mascara = new StringBuilder(17);
            StringBuilder gateway = new StringBuilder(17);

            // Lê valores - NÃO FAZ CHECAGEM DO RETORNO, MAS NÃO DEVEM RETORNAR VALORES NEGATIVOS!
            versao = ItsCamDLL.leVersaoFirmware(id);
            revisao = ItsCamDLL.leRevisaoFirmware(id);
            autoiris = ItsCamDLL.leAutoIris(id);
            daynight = ItsCamDLL.leModoDayNight(id);
            rotacao = ItsCamDLL.leRotacao(id);
            ItsCamDLL.leMac(id, mac);
            ItsCamDLL.leIp(id, ip);
            ItsCamDLL.leMascaraRede(id, mascara);
            ItsCamDLL.leGateway(id, gateway);

            // Versão/Revisão
            lVersao.Text = "Versão: " + versao.ToString();
            lRevisao.Text = "Revisão: " + revisao.ToString();

            // Autoíris
            if (autoiris == 0)
                // Sem autoíris
                rbSemAutoiris.Checked = true;
            else if (autoiris == 1)
                // Com autíris
                rbComAutoiris.Checked = true;
            else
                MessageBox.Show("Não foi possível ler a configuração da autoíris", "Erro na leitura", MessageBoxButtons.OK,MessageBoxIcon.Information);
            
            // Modo day/night
            if (daynight == 0)
                // Automático
                rbDayNightAutomatico.Checked = true;
            else if (daynight == 1)
                // Modo day
                rbDay.Checked = true;
            else if (daynight == 2)
                // Modo night
                rbNight.Checked = true;
            else
                MessageBox.Show("Não foi possível ler a configuração do modo day/night", "Erro na leitura", MessageBoxButtons.OK, MessageBoxIcon.Information);

            // Rotação
            if (rotacao == 0)
                // Imagem normal
                rbSemRotacao.Checked = true;
            else if (rotacao == 1)
                // Imagem rotacionada
                rb180.Checked = true;
            else
                MessageBox.Show("Não foi possível ler a configuração da rotação da imagem", "Erro na leitura", MessageBoxButtons.OK, MessageBoxIcon.Information);

            // MAC, IP, Máscara de Rede e Gateway
            lMac.Text = "MAC: " + mac.ToString();
            if (checkIp(ip.ToString()) && checkIp(mascara.ToString()) && checkIp(gateway.ToString()))
            {
                // Todos válidos
                tbIpAtual.Text = ip.ToString();
                tbMascaraRede.Text = mascara.ToString();
                tbGateway.Text = gateway.ToString();
            }
            else
                MessageBox.Show("Não foi possível ler a configurações de rede", "Erro na leitura", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
        /// <summary>
        /// Checa se o IP digitado é válido
        /// </summary>
        /// <param name="ip">IP no formato AAA.BBB.CCC.DDD</param>
        /// <returns>TRUE caso o IP fornecido é válido, FALSE caso contrário</returns>
        private bool checkIp(string ip)
        {
            // Cria o padrão
            string pattern = @"^([1-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])(\.([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$";
            // Cria o objeto para comparação
            Regex check = new Regex(pattern);
            // Testa e retorna
            return check.IsMatch(ip, 0);
        }
        /// <summary>
        /// Override de quando o usuário fechar a janela - força desconexão
        /// </summary>
        /// <param name="e"></param>
        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            base.OnFormClosing(e);

            // Mata threads
            toKillVideo = true;
            toKillFps = true;
            Thread.Sleep(500);

            // Fecha conexão
            ItsCamDLL.fecharConexaoItscam300(id);

            if (e.CloseReason == CloseReason.WindowsShutDown) return;
        }
        #endregion
    }

}
