#ifndef ITSCAM300_H
#define ITSCAM300_H

#ifdef LINUX
#include "compat_pumatronix.h"
#else
#include <windows.h>
#include <winsock.h>
#endif

#include "recebedados.h"

#ifdef CBUILDER
#define CALLING_CONVENTION __stdcall
#else
#define CALLING_CONVENTION __cdecl
#endif

class Itscam300{
private:
	static Itscam300 *ids[];
	int estado;
	int formatoProcessaDados;
	unsigned int contadorJPG;
	int tamanhoPacote;
	int byteCount;
	int tempInt;
	int status;
	int crcTab[256];
	int updateCrc(unsigned char *c, int numBytes);
	void initCrcTab();
	SOCKET sock;
	FilaRecebeDados *rd;
	RecebeDados *envelope;
	void (*funcaoLog)(const char *);
#ifdef LINUX
	pthread_t *thread1;
	pthread_t *thread2;
#else
	HANDLE thread1;
	HANDLE thread2;
#endif
	HANDLE semaforo;
	HANDLE semaforoFotos;
	HANDLE semSlots;
#ifndef LINUX
	HMODULE jidoshaDLL;
#endif
	int numeroFotos;
	int numeroFotosIO;
	int qualidadeFotoIO;
	int formatoFotoIO;
	int tempoLaco;
	char texto[64];
	char arqLog[2048];
	unsigned char macItscam[6];
	unsigned int timeoutIO;
	int versaoFirmware;
	unsigned int timeoutFoto;
	unsigned char auxPacote;
	unsigned char auxCrc;
	RecebeDados *config;
	RecebeDados *slots[128][16];
	unsigned char *imagem;
	int setaConfig(unsigned char, int, int, int);
	void processaDados(unsigned char *, int);
	int reqConfig(unsigned char config,unsigned char *parametros, int tparametros, int offset,int len, char *respostaEmString=NULL);
	int requisita(unsigned char **buf, int formato, int qualidade, bool video, int nfotos, int *tamanhos, const char *strAdicional=NULL);
	int requisitaIO(unsigned char **buf, int formato, int qualidade, int nfotos, int *tamanhos);
	int reqMultFotSemEspera(int nfotos, int formato, int qualidade, const char *str);
	void enviaDados(SOCKET s, char *data, int size);
	void log(const char *texto);
public:
	static unsigned char CABECALHO;
	Itscam300(const char *ip, int timeout);
	~Itscam300();
	void itscamThread();
	void keepAlive();
	static int criarItscam300(const char *ip, int timeout);
	static Itscam300 *getObject(int i){
		return ids[i];
	}

	int ocrJPEG(unsigned char *jpeg, char *placa);
	int setaFuncaoMensagem(void (*ptrFuncao)(const char *));

	int leRealceBorda();
	int leVersaoFirmware();
	int leHdr();
	int leTipoShutter();
	int leShutterFixo();
	int leShutterMaximo();
	int leTipoGanho();
	int leGanhoFixo();
	int leGanhoMaximo();
	int leModoTeste();
	int leNivelDesejado();
	int leValorNivel();
	int leValorGanho();
	int leValorShutter();
	int leSituacaoDayNight();
	int leFormatoFotoIO();
	int leQualidadeFotoIO();
	int lePesos(int *pesos);
	int leValorEntrada();
	int leRotacao();
	int leAutoIris();
	int leModoDayNight();
	int leModoOCR();
	int leRevisaoFirmware();
	int leTipoFlash();
	int leTrigger();
	int leTipoSaida();
	int leValorSaida();
	int leDelay();
	int leStatus();
	int leIp(char *ip);
	int leMascaraRede(char *mascara);
	int leConfigSerial(char *cfg);
	int leChavePublica(char *chv, int tamanhoChave);
	int leGateway(char *gateway);
	int leMac(char *mac);
	int lePorcentagemSegundoDisparo();
	int leNumeroFotos();
	int leNumeroFotosIO();
	int leSaturacao();
	int leBrilho();
	int leContraste();
	int leGamma();
	int leSombra();
	int leModelo();
	int leBalancoBranco(int *bb);
	int leFotoColorida();
	int leGanhoAlternativo();
	int leValorGanhoAlternativo();
	int leTriggerAlternativo();
	int leValorTriggerAlternativo();
	int leOcrAlternativo();
	int leValorOcrAlternativo();
	int leGammaAlternativo();
	int leValorGammaAlternativo();
	int leBalancoBrancoAlternativo();
	int leValorBalancoBrancoAlternativo(int *bb);
	int leBalancoBrancoAtual(int *bb);
	int leDelayCapturaDay();
	int leDelayCapturaNight();
	int lePosicaoZoom();
	int lePosicaoFoco();
	int leAutoFoco();
	int leDataAtual();
	int leHoraAtual();
	int lePortaServidor();
	int leIOVigia();
	int leDirIOVigia();
	int leTempoLigado();
	int leConfiguracoesImagem(unsigned char *img);
	int leGanhoSegundaFotoLuzVisivel();
	int leGanhoSegundaFotoLuzInfravermelha();
	int leFocoInfraVermelho();
	int leTempoEntreTriggers();
	int leTipoServidor();
	int leIntervaloTriggersPeriodicos();
	int leLimiarDayNight();
	int leLimiarNightDay();
	int leLimiarVisivelInfraLenteMotorizada();
	int leLimiarInfraVisivelLenteMotorizada();
	int setaLimiarDayNight(int limiar);
	int setaLimiarNightDay(int limiar);
	int setaLimiarVisivelInfraLenteMotorizada(int limiar);
	int setaLimiarInfraVisivelLenteMotorizada(int limiar);
	int setaNivelDesejado(int nivel);
	int setaRotacao(int rotacao180);
	int setaValorSaida(int saida);
	int setaHdr(int hdr);
	int setaDelay(int delay);
	int setaShutterFixo(int shutter);
	int setaShutterMaximo(int shutter);
	int setaGanhoFixo(int ganho);
	int setaGanhoMaximo(int ganho);
	int setaQualidadeFotoIO(int qualidade);
	int setaFlash(int flash);
	int setaTrigger(int trigger);
	int setaTipoSaida(int saida);
	int setaIp(char *ip);
	int setaMascaraRede(char *mascara);
	int setaGateway(char *gateway);
	int setaArquivoLog(char *arquivo);
	int setaTipoShutter(int tipo);
	int setaPortaServidor(int porta);
	int setaTipoGanho(int automatico);
	int setaModoTeste(int teste);
	int setaFormatoFotoIO(int formato);
	int setaLenteAutoIris(int autoiris);
	int setaModoOCR(int ocr);
	int setaModoDayNight(int daynight);
	int setaPesos(int *pesos);
	int setaDeadPixels(int *pixels);
	int setaPadrao();
	int reinicializaItscam();
	int setaNumeroFotos(int num);
	int setaPorcentagemSegundoDisparo(int porcentagem);
	int setaNumeroFotosIO(int num);
	int setaSaturacao(int saturacao);
	int setaBrilho(int brilho);
	int setaContraste(int contraste);
	int setaGamma(int gamma);
	int setaBalancoBranco(int *bb);
	int setaRealceBorda(int borda);
	int setaFotoColorida(int cor);
	int setaGanhoAlternativo(int tipo);
	int setaValorGanhoAlternativo(int ganho);
	int setaTriggerAlternativo(int tipo);
	int setaValorTriggerAlternativo(int trigger);
	int setaGammaAlternativo(int tipo);
	int setaValorGammaAlternativo(int gamma);
	int setaOcrAlternativo(int tipo);
	int setaValorOcrAlternativo(int ocr);
	int setaBalancoBrancoAlternativo(int tipo);
	int setaValorBalancoBrancoAlternativo(int *bb);
	int setaGanhoSegundaFotoLuzVisivel(int ganho);
	int setaGanhoSegundaFotoLuzInfravermelha(int ganho);
	int setaTempoEntreTriggers(int tempo);
	int setaFoco(int foco);
	int setaAutoFoco(int foco);
	int setaSombra(int sombra);
	int setaZoom(int zoom);
	int setaTipoServidor(int tipo);
	int setaConfigSerial(char *cfg);
	int setaDataAtual(int data);
	int setaHoraAtual(int hora);
	int setaTimeoutIO(int sec);
	int setaTimeoutFoto(int sec);
	int setaDelayCapturaDay(int delay);
	int setaDelayCapturaNight(int delay);
	int setaPosicaoZoom(int zoom);
	int setaPosicaoFoco(int foco);
	int setaFocoDayNight(int daynight);
	int setaMac(unsigned char *mac);
	int setaIOVigia(int io);
	int setaDirIOVigia(int dir);
	int setaIntervaloTriggersPeriodicos(int intervalo);
	int requisitaFoto(unsigned char *buf, int formato, int qualidade);
	int requisitaQuadroVideo(unsigned char *buf, int formato, int qualidade);
	int requisitaMultiplasFotos(unsigned char **buf, int nfotos, int *tamanho, int formato, int qualidade);
	int requisitaMultiplasFotosString(unsigned char **buf, int nfotos, int *tamanho, int formato, int qualidade, const char *str);
	int requisitaMultiplasFotosSemEspera(int nfotos, int formato, int qualidade);
	int requisitaMultiplasFotosSemEsperaString(int nfotos, int formato, int qualidade, const char *str);
	int requisitaFotoId(int slot, int idFoto, unsigned char *buf, int *tempo);
	int requisitaMultiplasFotosIOSemEspera(int nfotos, int formato, int qualidade);
	int salvarFoto(char *arquivo,int formato,int qualidade);
	int salvarMultiplasFotos(char **filename,int nFotos,int *tamFotos,int formato,int qualidade);
	int salvarMultiplasFotosString(char **filename,int nFotos,int *tamFotos,int formato,int qualidade, const char *str);
	int requisitaFotoIO(unsigned char *foto, int formato, int qualidade);
	int requisitaFotoTriggerContinuo(unsigned char *buf, int formato, int qualidade);
	int requisitaMultiplasFotosIO(unsigned char **foto,int nFotos,int *tamFotos,int formato,int qualidade);
	int salvarFotoIO(char *arquivo,int formato,int qualidade);
	int salvarMultiplasFotosIO(char **filename,int nFotos,int *tamFotos,int formato,int qualidade);
	int salvarFotoOcr(char *diretorio, int qualidade);
	int salvarFotoOcrTriggerContinuo(char *diretorio, int qualidade);
	int salvarFotoOcrIO(char *diretorio, int qualidade);
	int salvarMultiplasFotosOcr(char *diretorio, int nFotos, int qualidade);
	int salvarMultiplasFotosOcrString(char *diretorio, int nFotos, int qualidade, const char *str);
	int salvarMultiplasFotosOcrIO(char *diretorio, int nFotos, int qualidade);
	int salvarFotoTriggerContinuo(char *arquivo,int formato,int qualidade);
	int atualizarFirmware(unsigned char *firmware, int tamanho);
	int atualizarFirmwareSenha(unsigned char *firmware, int tamanho, unsigned short senha);
};

#endif
